@@ -15,7 +15,7 @@ from optparse import OptionParser, OptionGroup
 from PIL import Image as PImage
 from PIL import PngImagePlugin
 
-__version__ = '0.2.9.1'
+__version__ = '0.3'
 
 
 PADDING_REGEXP = re.compile("^(\d+-?){,3}\d+$")
@@ -46,6 +46,8 @@ DEFAULT_SETTINGS = {
     'png8': False,
     'ratios': '',
     'retina': False,
+    'imagemagick': False,
+    'imagemagickpath': 'convert',
     'separator': '-',
     'optipngpath': 'optipng',
     'optipng': False,
@@ -391,8 +393,11 @@ class Image(object):
         self.sprite = sprite
         self.filename, self.format = name.rsplit('.', 1)
 
-        pseudo = set(self.filename.split('_')).intersection(PSEUDO_CLASSES)
-        self.pseudo = ':%s' % list(pseudo)[-1] if pseudo else ''
+        if '_' in self.filename:
+            pseudo = set(self.filename.split('_')).intersection(PSEUDO_CLASSES)
+            self.pseudo = ':%s' % list(pseudo)[-1] if pseudo else ''
+        else:
+            self.pseudo = ''
 
         self.path = path or os.path.join(sprite.path, name)
 
@@ -549,8 +554,7 @@ class Image(object):
 
         # Handle CamelCase separator
         if separator == CAMELCASE_SEPARATOR:
-            namespace = [n.lower().capitalize() if i > 0 else n.lower() \
-                                            for i, n in enumerate(namespace)]
+            namespace = [n[:1].title() + n[1:] if i > 0 else n for i, n in enumerate(namespace)]
             separator = ''
 
         return separator.join(namespace)
@@ -786,19 +790,39 @@ class Sprite(object):
 
         # Loop all over the ratios and save one image for each one
         for ratio in ratios_to_process:
-
             sprite_image_path = self.image_path(ratio)
 
+            save_full_size = lambda: canvas.save(sprite_image_path, **kwargs)
+
             # If this canvas isn't the biggest one scale it using the ratio
             if self.max_ratio != ratio:
-                reduced_canvas = canvas.resize(
-                    (round_up((width / self.max_ratio) * ratio),
-                     round_up((height / self.max_ratio) * ratio)),
-                    PImage.ANTIALIAS)
-            else:
-                reduced_canvas = canvas
 
-            save = lambda: reduced_canvas.save(sprite_image_path, **kwargs)
+                def pil_save():
+                    reduced_canvas = canvas.resize(
+                            (round_up((width / self.max_ratio) * ratio),
+                             round_up((height / self.max_ratio) * ratio)),
+                            PImage.ANTIALIAS)
+                    reduced_canvas.save(sprite_image_path, **kwargs)
+
+                if self.config.imagemagick:
+                    def save():
+                        save_full_size()
+                        data = {'path': sprite_image_path,
+                                'imagemagickpath': self.config.imagemagickpath,
+                                'ratio': (100.0 / self.max_ratio) * ratio}
+                        command = ["%(imagemagickpath)s %(path)s -resize %(ratio)s%% %(path)s" % data]
+                        error = subprocess.call(command,
+                                                shell=True,
+                                                stdin=subprocess.PIPE,
+                                                stdout=subprocess.PIPE)
+                        if error:
+                            self.manager.log("Error: ImageMagic has failed, using Pillow to scale the sprite.")
+                            pil_save()
+                else:
+                    save = pil_save
+            else:
+                save = save_full_size
+
             save()
 
             # Optimize the image using optipng, if for some reason, it fails
@@ -848,9 +872,14 @@ class Sprite(object):
         # Write the hash line to the file.
         css_file.write(hash_line)
 
-        # get all the class names and join them
-        class_names = ',\n'.join(['.%s' % i.class_name for i in self.images \
-                                                  if ':' not in i.class_name])
+        # Get all the class names
+        class_names = ['.%s' % i.class_name for i in self.images]
+
+        # Exclude pseudo classes if the class is already in the list
+        class_names = [cn for cn in class_names if ':' not in cn or cn.rsplit(':')[0] not in class_names]
+
+        # Join class names
+        class_names = ',\n'.join(class_names)
 
         # add the global style for all the sprites for less bloat
         template = self.config.global_template.decode('unicode-escape')
@@ -1061,14 +1090,6 @@ class ConfigManager(object):
                 return value
         return self.defaults.get(name)
 
-    def _margin(self):
-        """Make retina sprites have at least 2px of margin to avoid noise
-        issues while resizing the sprite from 2x to 1x. If the --margin option
-        is present this setting will be ignored."""
-        if (self.find('retina') or self.find('ratios')) and int(self.find('margin')) < 2:
-            return 2
-        return int(self.find('margin'))
-
 
 class BaseManager(object):
 
@@ -1437,6 +1458,14 @@ def main():
             help="path to optipng (default: optipng)", metavar='PATH')
     parser.add_option_group(group)
 
+    group = OptionGroup(parser, "ImageMagick Options",
+                "You need to install ImageMagick before using these options")
+    group.add_option("--imagemagick", dest="imagemagick", action='store_true',
+            help="Use ImageMagick to scale down retina sprites instead of Pillow.")
+    group.add_option("--imagemagickpath", dest="imagemagickpath",
+            help="path to imagemagick (default: convert)", metavar='PATH')
+    parser.add_option_group(group)
+
     group = OptionGroup(parser, "Browser Cache Invalidation Options")
     group.add_option("--cachebuster", dest="cachebuster", action='store_true',
             help=("use the sprite's sha1 first 6 characters as a "
